/* -*-C-*- */

/*  kcond-i.h  */

/*
 * Author: Nikita Danilov <NikitaDanilov@yahoo.COM>
 * Keywords: kernel condition variables
 *
 * Copyright (C) 2000 Namesys (Hans Reiser)
 *
 * This file is part of kcond.
 *
 * Kcond is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

/* 
   This is implementation details of condition variables

   kcond.h contains declaration of module interface.

   kcond.c contains implementaion of core functions and module stuff.
   kcond-test.c contains trivial test.  
*/

#if !defined( __KCOND__I_H__ )
#define __KCOND__I_H__

/* $Id$ */


/* implementation details */

#include <linux/spinlock.h>
#include <linux/types.h>
#include <linux/list.h>

#include <asm/semaphore.h>

#define FORMAT( function, format, args... )							\
    function( "kcond : %s (%s:%i) [%i] " format "\n" ,				\
              __func__, debugFileId, __LINE__ , 					\
			  ( ( current != NULL ) ? current -> pid : -1 ) , ##args )

#define PRINT( format, args... ) FORMAT( printk, format, ##args )
#define PANIC( format, args... ) FORMAT( panic, format, ##args )

#if defined( KCOND_DEBUG )

#define ASSERT( E )                ( { if( !( E ) ) { PANIC( #E ); } } )
#define DEBUG( format, args... )   PRINT( format, ##args )
#define DEXP( E )                  E

#else

#define ASSERT( E )
#define DEBUG( format, args... )
#define DEXP( E )

#endif

#if defined( KCOND_STATS )
/* lisp-p */
#define SEXP( E )                  E
#else
#define SEXP( E )
#endif
#define TRACE  DEBUG( "trace" )

struct debug_info
{
  DEXP( void *return_address );
  DEXP( pid_t pid );
};

struct kcond_queue_link;
struct kcond
{
  spinlock_t lock;
  struct kcond_queue_link *queue;
  struct debug_info dinfo;
};

struct kcond_queue_link
{
  struct kcond_queue_link *next;
  kcond_synchro_t          synchro;
  void                    *generic;
  kcond_pred_t            *pred;
  struct semaphore         wait;
  int                      enqueued;
  struct debug_info        dinfo;
};

struct kcond_synchro_fun
{
  int   (*fun)( kcond_t *cvar, void *data );
  void   *data;
};

struct  kcond_pred
{
  int   (*pred)( kcond_t *cvar, void *data );
  void   *data;
};

/*
 * $Log$
 */

/* __KCOND__I_H__ */
#endif


