/* -*-C-*- */

/*  kcond.h  */

/*
 * Author: Nikita Danilov <NikitaDanilov@yahoo.COM>
 * Keywords: kernel condition variables
 *
 * Copyright (C) 2000 Namesys (Hans Reiser)
 *
 * This file is part of kcond.
 *
 * Kcond is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

/* 
   This is implementation of condition variables (see
   e.g., pthread_cond_init(3) or any text-book on multithreading).

   Condition variables are implemented on top of Linux kernel
   semaphores and spin-locks to protect internal fields.

   If KCOND_STATS is defined, some statistics are gathered and are
   available through /proc/kcond.

   Implementation is rather strait-forward.

   kcond.c contains implementaion of core functions and module stuff.
   kcond-test.c contains trivial test.  
*/

#if !defined( __KCOND_H__ )
#define __KCOND_H__

/* $Id$ */

#include <linux/version.h>
#include <linux/kernel.h>

/* this all is for our mighty kernel only */
#if defined( __KERNEL__ )

#if defined( CONFIG_SMP )

#define KCOND_DEBUG
#define KCOND_STATS

struct semaphore;

/* public interface */

struct kcond;
typedef struct kcond kcond_t;

typedef enum { sema, rw, fun, spin } kcond_synchro_t;
struct  kcond_synchro_fun;
typedef struct kcond_synchro_fun kcond_synchro_fun_t;
struct  kcond_pred;
typedef struct kcond_pred kcond_pred_t;

extern kcond_t *kcond_init( kcond_t *cvar );
extern kcond_t  kcond_static_init( void );
extern int      kcond_destroy( kcond_t *cvar );

extern int      kcond_wait( kcond_t *cvar, struct semaphore *sem );
extern int      kcond_wait_with( kcond_t *cvar, 
								 kcond_synchro_t synchro_type, void *obj,
								 kcond_pred_t *pred );

extern int      kcond_signal( kcond_t *cvar );
extern int      kcond_broadcast( kcond_t *cvar );

extern int      kcond_are_waiters( kcond_t *cvar );

extern int      kcond_test( void );
extern void     kcond_print( kcond_t *cvar );

#include "kcond-i.h"

/* CONFIG_SMP */
#else
#warning "kcond: condition variables require SMP support"
#endif

/* __KERNEL__ */
#endif

/*
 * $Log$
 */

/* __KCOND_H__ */
#endif

